@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 
@@ -59,9 +59,16 @@ stetmesh::TmComp::TmComp(std::string const & id, Tetmesh * container,
 {
     if (pTetmesh == 0)
     {
-    std::ostringstream os;
-    os << "No mesh provided to Comp initializer function";
-    throw steps::ArgErr(os.str());
+    	std::ostringstream os;
+    	os << "No mesh provided to TmComp initializer function";
+    	throw steps::ArgErr(os.str());
+    }
+
+    if (tets.size() == 0)
+    {
+    	std::ostringstream os;
+    	os << "No tetrahedrons provided to TmComp initializer function";
+    	throw steps::ArgErr(os.str());
     }
 
 	// The maximum tetrahedron index in tetrahedral mesh
