@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2010Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 // STL headers.
@@ -80,6 +80,20 @@ ssolver::Diffdef::~Diffdef(void)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+void ssolver::Diffdef::checkpoint(std::fstream & cp_file)
+{
+    cp_file.write((char*)&pDcst, sizeof(double));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+void ssolver::Diffdef::restore(std::fstream & cp_file)
+{
+    cp_file.read((char*)&pDcst, sizeof(double));
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 void ssolver::Diffdef::setup(void)
 {
 	assert (pSetupdone == false);
