@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2010Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_SOLVER_SREACDEF_HPP
@@ -34,6 +34,7 @@
 
 // STL headers.
 #include <string>
+#include <fstream>
 
 // STEPS headers.
 #include "../common.h"
@@ -76,6 +77,15 @@ public:
     /// Destructor
 	~SReacdef(void);
 
+    ////////////////////////////////////////////////////////////////////////
+    // CHECKPOINTING
+    ////////////////////////////////////////////////////////////////////////
+    /// checkpoint data
+    void checkpoint(std::fstream & cp_file);
+
+    /// restore data
+    void restore(std::fstream & cp_file);
+
     ////////////////////////////////////////////////////////////////////////
     // DATA ACCESS: SURFACE REACTION RULE
     ////////////////////////////////////////////////////////////////////////
@@ -148,6 +158,12 @@ public:
     ///
     bool reqOutside(void) const;
 
+    /// Return true if this reaction only involves surface species,
+    /// nothing in a volume at all. In that case the reaction constant
+    /// should be treated in 2D
+    inline bool surf_surf(void) const
+    { return pSurface_surface; }
+
     ////////////////////////////////////////////////////////////////////////
     // DATA ACCESS: STOICHIOMETRY
     ////////////////////////////////////////////////////////////////////////
@@ -234,6 +250,8 @@ private:
 
 	bool								pSetupdone;
 
+	// Store whether this surface reaction is 2D or not
+	bool 								pSurface_surface;
 
     /// Does the left-hand side of the stoichiometry involve molecules
     /// on the inside or on the outside?
