@@ -1,7 +1,6 @@
-
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2010Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -24,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 // STL headers.
@@ -47,6 +46,7 @@
 #include "reacdef.hpp"
 #include "sreacdef.hpp"
 #include "diffdef.hpp"
+#include "diffboundarydef.hpp"
 
 NAMESPACE_ALIAS(steps::solver, ssolver);
 
@@ -64,6 +64,7 @@ ssolver::Statedef::Statedef(steps::model::Model * m, steps::wm::Geom * g, steps:
 , pReacdefs()
 , pSReacdefs()
 , pDiffdefs()
+, pDiffBoundarydefs()
 
 {
     assert(pModel != 0);
@@ -120,6 +121,17 @@ ssolver::Statedef::Statedef(steps::model::Model * m, steps::wm::Geom * g, steps:
     	pPatchdefs.push_back(patchdef);
     }
 
+    if (steps::tetmesh::Tetmesh * tetmesh = dynamic_cast<steps::tetmesh::Tetmesh *>(pGeom))
+    {
+    	uint ndiffbs = tetmesh->_countDiffBoundaries();
+    	for (uint dbidx = 0; dbidx < ndiffbs; ++dbidx)
+    	{
+    		ssolver::DiffBoundarydef * diffboundarydef = new DiffBoundarydef(this, dbidx, tetmesh->_getDiffBoundary(dbidx));
+    		assert (diffboundarydef != 0);
+    		pDiffBoundarydefs.push_back(diffboundarydef);
+    	}
+    }
+
     // Now setup all the def objects. This can't be achieved purely with		/// check the order
     // the constructors, e.g.  a patch may need to add species from its
     // surface reactions to inner, outer comp
@@ -142,6 +154,10 @@ ssolver::Statedef::Statedef(steps::model::Model * m, steps::wm::Geom * g, steps:
     	(*c)->setup_indices();
     for (PatchdefPVecI p = pPatchdefs.begin(); p != pPatchdefs.end(); ++p)
     	(*p)->setup_indices();
+
+    for (DiffBoundaryDefPVecI db = pDiffBoundarydefs.begin(); db != pDiffBoundarydefs.end(); ++db)
+    	(*db)->setup();
+
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -157,6 +173,9 @@ ssolver::Statedef::~Statedef()
 	PatchdefPVecCI p_end = pPatchdefs.end();
 	for (PatchdefPVecCI p = pPatchdefs.begin(); p != p_end; ++p) delete *p;
 
+	DiffBoundarydefPVecCI db_end = pDiffBoundarydefs.end();
+	for (DiffBoundarydefPVecCI db = pDiffBoundarydefs.begin(); db != db_end; ++db) delete *db;
+
 	ReacdefPVecCI r_end = pReacdefs.end();
 	for (ReacdefPVecCI r = pReacdefs.begin(); r != r_end; ++r) delete *r;
 
@@ -169,6 +188,92 @@ ssolver::Statedef::~Statedef()
 
 ////////////////////////////////////////////////////////////////////////////////
 
+void ssolver::Statedef::checkpoint(std::fstream & cp_file)
+{
+    cp_file.write((char*)&pTime, sizeof(double));
+    cp_file.write((char*)&pNSteps, sizeof(uint));
+
+	SpecdefPVecCI s_end = pSpecdefs.end();
+	for (SpecdefPVecCI s = pSpecdefs.begin(); s != s_end; ++s) {
+        (*s)->checkpoint(cp_file);
+    }
+
+	CompdefPVecCI c_end = pCompdefs.end();
+	for (CompdefPVecCI c = pCompdefs.begin(); c != c_end; ++c) {
+        (*c)->checkpoint(cp_file);
+    }
+
+	PatchdefPVecCI p_end = pPatchdefs.end();
+	for (PatchdefPVecCI p = pPatchdefs.begin(); p != p_end; ++p) {
+        (*p)->checkpoint(cp_file);
+    }
+
+	DiffBoundarydefPVecCI db_end = pDiffBoundarydefs.end();
+	for (DiffBoundarydefPVecCI db = pDiffBoundarydefs.begin(); db != db_end; ++db) {
+        (*db)->checkpoint(cp_file);
+    }
+
+	ReacdefPVecCI r_end = pReacdefs.end();
+	for (ReacdefPVecCI r = pReacdefs.begin(); r != r_end; ++r) {
+        (*r)->checkpoint(cp_file);
+    }
+
+	SReacdefPVecCI sr_end = pSReacdefs.end();
+	for (SReacdefPVecCI sr = pSReacdefs.begin(); sr != sr_end; ++sr) {
+        (*sr)->checkpoint(cp_file);
+    }
+
+	DiffdefPVecCI d_end = pDiffdefs.end();
+	for (DiffdefPVecCI d = pDiffdefs.begin(); d != d_end; ++d) {
+        (*d)->checkpoint(cp_file);
+    }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+void ssolver::Statedef::restore(std::fstream & cp_file)
+{
+    cp_file.read((char*)&pTime, sizeof(double));
+    cp_file.read((char*)&pNSteps, sizeof(uint));
+
+	SpecdefPVecCI s_end = pSpecdefs.end();
+	for (SpecdefPVecCI s = pSpecdefs.begin(); s != s_end; ++s) {
+        (*s)->restore(cp_file);
+    }
+
+	CompdefPVecCI c_end = pCompdefs.end();
+	for (CompdefPVecCI c = pCompdefs.begin(); c != c_end; ++c) {
+        (*c)->restore(cp_file);
+    }
+
+	PatchdefPVecCI p_end = pPatchdefs.end();
+	for (PatchdefPVecCI p = pPatchdefs.begin(); p != p_end; ++p) {
+        (*p)->restore(cp_file);
+    }
+
+	DiffBoundarydefPVecCI db_end = pDiffBoundarydefs.end();
+	for (DiffBoundarydefPVecCI db = pDiffBoundarydefs.begin(); db != db_end; ++db) {
+        (*db)->restore(cp_file);
+    }
+
+	ReacdefPVecCI r_end = pReacdefs.end();
+	for (ReacdefPVecCI r = pReacdefs.begin(); r != r_end; ++r) {
+        (*r)->restore(cp_file);
+    }
+
+	SReacdefPVecCI sr_end = pSReacdefs.end();
+	for (SReacdefPVecCI sr = pSReacdefs.begin(); sr != sr_end; ++sr) {
+        (*sr)->restore(cp_file);
+    }
+
+	DiffdefPVecCI d_end = pDiffdefs.end();
+	for (DiffdefPVecCI d = pDiffdefs.begin(); d != d_end; ++d) {
+        (*d)->restore(cp_file);
+    }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 ssolver::Compdef * ssolver::Statedef::compdef(uint gidx) const
 {
     assert(gidx < pCompdefs.size());
@@ -419,6 +524,65 @@ uint ssolver::Statedef::getDiffIdx(steps::model::Diff * diff) const
 
 ////////////////////////////////////////////////////////////////////////////////
 
+ssolver::DiffBoundarydef * ssolver::Statedef::diffboundarydef(uint gidx) const
+{
+	assert(gidx < pDiffBoundarydefs.size());
+	return pDiffBoundarydefs[gidx];
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+uint ssolver::Statedef::getDiffBoundaryIdx(std::string const & d) const
+{
+	uint maxdidx = pDiffBoundarydefs.size();
+	if (steps::tetmesh::Tetmesh * tetmesh = dynamic_cast<steps::tetmesh::Tetmesh *>(pGeom))
+	{
+		assert (maxdidx == tetmesh->_countDiffBoundaries());
+		uint didx = 0;
+		while(didx < maxdidx)
+		{
+			if (d == tetmesh->_getDiffBoundary(didx)->getID()) return didx;
+			++didx;
+		}
+		std::ostringstream os;
+		os << "Geometry does not contain diff boundary with string identifier '" << d <<"'.";
+		throw steps::ArgErr(os.str());
+	}
+	else
+	{
+		std::ostringstream os;
+		os << "Diffusion boundary methods not available with well-mixed geometry";
+		throw steps::ArgErr(os.str());
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+uint ssolver::Statedef::getDiffBoundaryIdx(steps::tetmesh::DiffBoundary * diffb) const
+{
+	uint maxdidx = pDiffBoundarydefs.size();
+	if (steps::tetmesh::Tetmesh * tetmesh = dynamic_cast<steps::tetmesh::Tetmesh *>(pGeom))
+	{
+		assert (maxdidx == tetmesh->_countDiffBoundaries());
+		uint didx = 0;
+		while(didx < maxdidx)
+		{
+			if (diffb == tetmesh->_getDiffBoundary(didx)) return didx;
+			++didx;
+		}
+		// Argument should be valid so we should not get here
+		assert(false);
+	}
+	else
+	{
+		std::ostringstream os;
+		os << "Diffusion boundary methods not available with well-mixed geometry";
+		throw steps::ArgErr(os.str());
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 void ssolver::Statedef::setTime(double t)
 {
 	assert (t >= 0.0);
