@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2010Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_SOLVER_STATEDEF_HPP
@@ -35,6 +35,7 @@
 // STL headers.
 #include <string>
 #include <vector>
+#include <fstream>
 
 // STEPS headers.
 #include "../common.h"
@@ -43,6 +44,7 @@
 #include "../rng/rng.hpp"
 #include "api.hpp"
 #include "../geom/patch.hpp"
+#include "../geom/diffboundary.hpp"
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -56,6 +58,7 @@ class Specdef;
 class Reacdef;
 class SReacdef;
 class Diffdef;
+class DiffBoundarydef;
 
 // Auxiliary declarations.
 
@@ -89,6 +92,11 @@ typedef std::vector<DiffdefP>            DiffdefPVec;
 typedef DiffdefPVec::iterator            DiffdefPVecI;
 typedef DiffdefPVec::const_iterator      DiffdefPVecCI;
 
+typedef DiffBoundarydef *                    DiffBoundarydefP;
+typedef std::vector<DiffBoundarydefP>        DiffBoundarydefPVec;
+typedef DiffBoundarydefPVec::iterator        DiffBoundarydefPVecI;
+typedef DiffBoundarydefPVec::const_iterator  DiffBoundarydefPVecCI;
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Defined State
 class Statedef
@@ -121,6 +129,15 @@ public:
     /// Destructor
     ~Statedef(void);
 
+    ////////////////////////////////////////////////////////////////////////
+    // CHECKPOINTING
+    ////////////////////////////////////////////////////////////////////////
+    /// checkpoint data
+    void checkpoint(std::fstream & cp_file);
+
+    /// restore data
+    void restore(std::fstream & cp_file);
+
     ////////////////////////////////////////////////////////////////////////
     // DATA ACCESS: COMPARTMENTS
     ////////////////////////////////////////////////////////////////////////
@@ -286,6 +303,39 @@ public:
     /// \exception Throw exception if model does not contain diff with this identifier.
     uint getDiffIdx(steps::model::Diff * diff) const;
 
+    ////////////////////////////////////////////////////////////////////////
+    // DATA ACCESS: DIFFUSION BOUNDARY
+    ////////////////////////////////////////////////////////////////////////
+
+    /// Return the total number of diffusion boundaries in the simulation state.
+    uint countDiffBoundaries(void) const
+    { return pDiffBoundarydefs.size(); }
+
+    /// Return pointer to DiffBoundarydef object specified by global index argument.
+    ///
+    /// \param gidx Global index of the diffusion boundary.
+    DiffBoundarydef * diffboundarydef(uint gidx) const;
+
+    /// Return the global index of diffusion boundary identified by string argument.
+    ///
+    /// \param d Name of the diffusion boundary.
+    /// \exception Throw exception if geometry does not contain diff boundary with this identifier.
+    uint getDiffBoundaryIdx(std::string const & d) const;
+
+    /// Return the global index of diffusion boundary identified by object argument.
+    ///
+    /// \param diff Pointer to the diffusion boundary object.
+    /// \exception Throw exception if geometry does not contain diff boundary with this identifier.
+    uint getDiffBoundaryIdx(steps::tetmesh::DiffBoundary * diffb) const;
+
+    /// Return the beginning iterator of the Diffusion Boundary objects.
+    std::vector<DiffBoundarydef *>::const_iterator bgnDiffBoundary(void) const
+    { return pDiffBoundarydefs.begin(); }
+
+    /// Return the end iterator of the Diffusion Boundary objects.
+    std::vector<DiffBoundarydef *>::const_iterator endDiffBoundary(void) const
+    { return pDiffBoundarydefs.end(); }
+
     ////////////////////////////////////////////////////////////////////////
     // DATA ACCESS: STATE
     ////////////////////////////////////////////////////////////////////////
@@ -332,7 +382,7 @@ public:
     /// Return current simulation time step.
     inline uint nsteps(void) const
     { return pNSteps; }
-    
+
     inline void setNSteps(uint nsteps)
     { pNSteps = nsteps; }
 
@@ -354,6 +404,7 @@ private:
 	std::vector<Reacdef *>              pReacdefs;
 	std::vector<SReacdef *>             pSReacdefs;
 	std::vector<Diffdef *>              pDiffdefs;
+	std::vector<DiffBoundarydef *>      pDiffBoundarydefs;
 
 };
 
