@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 
@@ -63,6 +63,18 @@ stex::Comp::~Comp(void)
 
 ////////////////////////////////////////////////////////////////////////////////
 
+void stex::Comp::checkpoint(std::fstream & cp_file)
+{
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
+void stex::Comp::restore(std::fstream & cp_file)
+{
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 void stex::Comp::addTet(stex::Tet * tet)
 {
 	assert (tet->compdef() == def());
