@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_TETEXACT_DIFF_HPP
@@ -36,6 +36,7 @@
 #include <map>
 #include <string>
 #include <vector>
+#include <fstream>
 
 // STEPS headers.
 #include "../common.h"
@@ -70,10 +71,22 @@ public:
     Diff(steps::solver::Diffdef * ddef, steps::tetexact::Tet * tet);
     ~Diff(void);
 
+    ////////////////////////////////////////////////////////////////////////
+    // CHECKPOINTING
+    ////////////////////////////////////////////////////////////////////////
+    /// checkpoint data
+    void checkpoint(std::fstream & cp_file);
+
+    /// restore data
+    void restore(std::fstream & cp_file);
+
     ////////////////////////////////////////////////////////////////////////
     // VIRTUAL INTERFACE METHODS
     ////////////////////////////////////////////////////////////////////////
 
+    inline steps::solver::Diffdef * def(void) const
+    { return pDiffdef; }
+
     inline double dcst(void) const
     { return pDcst; }
     void setDcst(double d);
@@ -89,8 +102,9 @@ public:
 
     ////////////////////////////////////////////////////////////////////////
 
-    //inline steps::solver::Reacdef * defr(void) const
-    //{ return pReacdef; }
+    void setDiffBndActive(uint i, bool active);
+
+    bool getDiffBndActive(uint i) const;
 
     ////////////////////////////////////////////////////////////////////////
 
@@ -101,6 +115,12 @@ private:
     steps::solver::Diffdef            * pDiffdef;
     steps::tetexact::Tet              * pTet;
     std::vector<uint>                   pUpdVec[4];
+
+    // Storing the species local index for each neighbouring tet: Needed
+    // because neighbours may belong to different compartments
+    // and therefore have different spec indices
+    int 							    pNeighbCompLidx[4];
+
     /// Properly scaled diffusivity constant.
     double                              pScaledDcst;
     // Compartmental dcst. Stored for convenience
@@ -108,6 +128,12 @@ private:
     /// Used in selecting which directory the molecule should go.
     double                              pCDFSelector[3];
 
+    // A flag to see if the species can move between compartments
+    bool 								pDiffBndActive[4];
+
+    // Flags to store if a direction is a diffusion boundary direction
+    bool 							    pDiffBndDirection[4];
+
     ////////////////////////////////////////////////////////////////////////
 
 };
