@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_TETEXACT_TETEXACT_HPP
@@ -48,6 +48,7 @@
 #include "kproc.hpp"
 #include "comp.hpp"
 #include "patch.hpp"
+#include "diffboundary.hpp"
 
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -105,8 +106,10 @@ public:
     void run(double endtime);
     void advance(double adv);
     void step(void);
-    
-    
+
+    void checkpoint(std::string const & file_name);
+    void restore(std::string const & file_name);
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      GENERAL
@@ -118,16 +121,16 @@ public:
     { return pA0; }
 
     uint getNSteps(void) const;
-    
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      ADVANCE
     //      Developer only
-    //////////////////////////////////////////////////////////////////////// 
-    
-    void setTime(double time);    
+    ////////////////////////////////////////////////////////////////////////
+
+    void setTime(double time);
     void setNSteps(uint nsteps);
-    
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      COMPARTMENT
@@ -195,6 +198,14 @@ public:
     uint _getPatchSReacExtent(uint pidx, uint ridx) const;
     void _resetPatchSReacExtent(uint pidx, uint ridx);
 
+    ////////////////////////////////////////////////////////////////////////
+    // SOLVER STATE ACCESS:
+    //      DIFFUSION BOUNDARIES
+    ////////////////////////////////////////////////////////////////////////
+
+    void _setDiffBoundaryDiffusionActive(uint dbidx, uint sidx, bool act);
+    bool _getDiffBoundaryDiffusionActive(uint dbidx, uint sidx) const;
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      TETRAHEDRAL VOLUME ELEMENTS
@@ -206,6 +217,8 @@ public:
     double _getTetCount(uint tidx, uint sidx) const;
     void _setTetCount(uint tidx, uint sidx, double n);
 
+    bool _getTetSpecDefined(uint tidx, uint sidx) const;
+
     double _getTetAmount(uint tidx, uint sidx) const;
     void _setTetAmount(uint tidx, uint sidx, double m);
 
@@ -243,6 +256,8 @@ public:
     double _getTriArea(uint tidx) const;
     void _setTriArea(uint tidx, double area);
 
+    bool _getTriSpecDefined(uint tidx, uint sidx) const;
+
     double _getTriCount(uint tidx, uint sidx) const;
     void _setTriCount(uint tidx, uint sidx, double n);
 
@@ -286,6 +301,8 @@ private:
 
 	uint _addPatch(steps::solver::Patchdef * pdef);
 
+	uint _addDiffBoundary(steps::solver::DiffBoundarydef * dbdef);
+
 	void _addTet(uint tetidx, steps::tetexact::Comp * comp, double vol, double a1,
 			     double a2, double a3, double a4, double d1, double d2,
 			     double d3, double d4, int tet0, int tet1, int tet2, int tet3);
@@ -340,7 +357,13 @@ private:
     inline steps::tetexact::Patch * _patch(uint pidx) const
     { return pPatches[pidx]; }
 
+    std::vector<steps::tetexact::DiffBoundary *> pDiffBoundaries;
+    inline steps::tetexact::DiffBoundary * _diffboundary(uint dbidx) const
+    { return pDiffBoundaries[dbidx]; }
+
     std::vector<steps::tetexact::Tet *>        pTets;
+    inline steps::tetexact::Tet * _tet(uint tidx) const
+    { return pTets[tidx]; }
 
     std::vector<steps::tetexact::Tri *>        pTris;
 
