@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 
@@ -35,6 +35,7 @@
 #include <cassert>
 #include <iostream>
 #include <sstream>
+#include <fstream>
 
 // STEPS headers.
 #include "../common.h"
@@ -163,7 +164,7 @@ void swmrk4::Wmrk4::reset(void)
 	uint comps = statedef()->countComps();
 	for (uint i=0; i < comps; ++i) statedef()->compdef(i)->reset();
 	uint patches = statedef()->countPatches();
-	for (uint i=0; i < patches; ++i) statedef()->compdef(i)->reset();
+	for (uint i=0; i < patches; ++i) statedef()->patchdef(i)->reset();
 	statedef()->resetTime();
 	// recompute flags and counts vectors in Wmrk4 object
 	_refill();
@@ -230,6 +231,106 @@ double swmrk4::Wmrk4::getTime(void) const
 
 ///////////////////////////////////////////////////////////////////////////////
 
+void swmrk4::Wmrk4::checkpoint(std::string const & file_name)
+{
+	std::fstream cp_file;
+
+    cp_file.open(file_name.c_str(),
+                std::fstream::out | std::fstream::binary | std::fstream::trunc);
+    double state_buffer[3];
+    state_buffer[0] = static_cast<double>(pSpecs_tot);
+    state_buffer[1] = static_cast<double>(pReacs_tot);
+    state_buffer[2] = pDT;
+
+    cp_file.write((char*)&state_buffer, sizeof(double) * 3);
+
+    cp_file.write((char*)pReacMtx, sizeof(uint) * pReacs_tot * pSpecs_tot);
+
+    cp_file.write((char*)pUpdMtx, sizeof(int) * pReacs_tot * pSpecs_tot);
+
+    cp_file.write((char*)pDyDxlhs, sizeof(double) * pSpecs_tot * pReacs_tot);
+
+    cp_file.write((char*)&pCcst.front(), sizeof(double) * pCcst.size());
+
+    cp_file.write((char*)&pVals.front(), sizeof(double) * pVals.size());
+
+    cp_file.write((char*)&pSFlags.front(), sizeof(uint) * pSFlags.size());
+
+    cp_file.write((char*)&pRFlags.front(), sizeof(uint) * pRFlags.size());
+
+    cp_file.write((char*)&pNewVals.front(), sizeof(double) * pNewVals.size());
+
+    cp_file.write((char*)&pDyDx.front(), sizeof(double) * pDyDx.size());
+
+    cp_file.write((char*)&yt.front(), sizeof(double) * yt.size());
+
+    cp_file.write((char*)&dyt.front(), sizeof(double) * dyt.size());
+
+    cp_file.write((char*)&dym.front(), sizeof(double) * dym.size());
+
+	statedef()->checkpoint(cp_file);
+
+    cp_file.close();
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
+void swmrk4::Wmrk4::restore(std::string const & file_name)
+{
+	std::fstream cp_file;
+
+    cp_file.open(file_name.c_str(),
+                std::fstream::in | std::fstream::binary);
+
+    cp_file.seekg(0);
+    double state_buffer[3];
+    cp_file.read((char*)&state_buffer, sizeof(double) * 3);
+
+    if (static_cast<uint>(state_buffer[0]) != pSpecs_tot) {
+        std::ostringstream os;
+        os << "checkpoint data mismatch with simulator parameters: pSpecs_tot.";
+        throw steps::ArgErr(os.str());
+    }
+
+    if (static_cast<uint>(state_buffer[1]) != pReacs_tot) {
+        std::ostringstream os;
+        os << "checkpoint data mismatch with simulator parameters: pReacs_tot.";
+        throw steps::ArgErr(os.str());
+    }
+
+    pDT = state_buffer[2];
+
+    cp_file.read((char*)pReacMtx, sizeof(uint) * pReacs_tot * pSpecs_tot);
+
+    cp_file.read((char*)pUpdMtx, sizeof(int) * pReacs_tot * pSpecs_tot);
+
+    cp_file.read((char*)pDyDxlhs, sizeof(double) * pSpecs_tot * pReacs_tot);
+
+    cp_file.read((char*)&pCcst.front(), sizeof(double) * pCcst.size());
+
+    cp_file.read((char*)&pVals.front(), sizeof(double) * pVals.size());
+
+    cp_file.read((char*)&pSFlags.front(), sizeof(uint) * pSFlags.size());
+
+    cp_file.read((char*)&pRFlags.front(), sizeof(uint) * pRFlags.size());
+
+    cp_file.read((char*)&pNewVals.front(), sizeof(double) * pNewVals.size());
+
+    cp_file.read((char*)&pDyDx.front(), sizeof(double) * pDyDx.size());
+
+    cp_file.read((char*)&yt.front(), sizeof(double) * yt.size());
+
+    cp_file.read((char*)&dyt.front(), sizeof(double) * dyt.size());
+
+    cp_file.read((char*)&dym.front(), sizeof(double) * dym.size());
+
+	statedef()->restore(cp_file);
+
+    cp_file.close();
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 double swmrk4::Wmrk4::_getCompVol(uint cidx) const
 {
 	assert(cidx < statedef()->countComps());
@@ -260,7 +361,12 @@ double swmrk4::Wmrk4::_getCompCount(uint cidx, uint sidx) const
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint slidx = comp->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 	return comp->pools()[slidx];
 }
 
@@ -274,7 +380,12 @@ void swmrk4::Wmrk4::_setCompCount(uint cidx, uint sidx, double n)
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint slidx = comp->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 	comp->setCount(slidx, n);
 	// easier to recompute all counts with _refill method
 	_refill();					/// may be a better way of doing this
@@ -293,7 +404,7 @@ double swmrk4::Wmrk4::_getCompAmount(uint cidx, uint sidx) const
 
 void swmrk4::Wmrk4::_setCompAmount(uint cidx, uint sidx, double a)
 {
-	assert(a > 0.0);
+	assert(a >= 0.0);
 	// convert amount in mols to number of molecules
 	double a2 = a * steps::math::AVOGADRO;
 	// the following method does all the necessary argument checking
@@ -334,8 +445,12 @@ bool swmrk4::Wmrk4::_getCompClamped(uint cidx, uint sidx) const
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lsidx = comp->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return false;
-
+	if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
     return comp->clamped(lsidx);
 }
 
@@ -348,7 +463,12 @@ void swmrk4::Wmrk4::_setCompClamped(uint cidx, uint sidx, bool b)
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lsidx = comp->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     comp->setClamped(lsidx, b);
 
@@ -365,7 +485,12 @@ double swmrk4::Wmrk4::_getCompReacK(uint cidx, uint ridx) const
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return comp->kcst(lridx);
 }
@@ -380,7 +505,12 @@ void swmrk4::Wmrk4::_setCompReacK(uint cidx, uint ridx, double kf)
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	comp->setKcst(lridx, kf);
 
@@ -397,7 +527,12 @@ bool swmrk4::Wmrk4::_getCompReacActive(uint cidx, uint ridx) const
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return false;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return (comp->active(lridx));
 }
@@ -411,8 +546,12 @@ void swmrk4::Wmrk4::_setCompReacActive(uint cidx, uint ridx, bool a)
 	Compdef * comp = statedef()->compdef(cidx);
 	assert(comp != 0);
 	uint lridx = comp->reacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
-
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Reaction undefined in compartment.\n";
+		throw steps::ArgErr(os.str());
+	}
 	comp->setActive(lridx, a);
 
 	// copy flags to this solver
@@ -448,8 +587,12 @@ double swmrk4::Wmrk4::_getPatchCount(uint pidx, uint sidx) const
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint slidx = patch->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return 0.0;
-
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 	return patch->pools()[slidx];
 }
 
@@ -462,7 +605,13 @@ void swmrk4::Wmrk4::_setPatchCount(uint pidx, uint sidx, double n)
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint slidx = patch->specG2L(sidx);
-	if (slidx == ssolver::LIDX_UNDEFINED) return;
+	if (slidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
+
 	patch->setCount(slidx, n);
 	// easier to recompute all counts with _refill method
 	_refill();
@@ -481,7 +630,7 @@ double swmrk4::Wmrk4::_getPatchAmount(uint pidx, uint sidx) const
 
 void swmrk4::Wmrk4::_setPatchAmount(uint pidx, uint sidx, double a)
 {
-	assert(a > 0.0);
+	assert(a >= 0.0);
 	// convert amount in mols to number of molecules
 	double a2 = a * steps::math::AVOGADRO;
 	// the following method does all the necessary argument checking
@@ -497,7 +646,12 @@ bool swmrk4::Wmrk4::_getPatchClamped(uint pidx, uint sidx) const
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lsidx = patch->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return false;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     return patch->clamped(lsidx);
 }
@@ -511,7 +665,12 @@ void swmrk4::Wmrk4::_setPatchClamped(uint pidx, uint sidx, bool buf)
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lsidx = patch->specG2L(sidx);
-    if (lsidx == ssolver::LIDX_UNDEFINED) return;
+    if (lsidx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Species undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
     patch->setClamped(lsidx, buf);
 
@@ -528,8 +687,12 @@ double swmrk4::Wmrk4::_getPatchSReacK(uint pidx, uint ridx) const
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return 0.0;
-
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 	return patch->kcst(lridx);
 }
 
@@ -543,7 +706,12 @@ void swmrk4::Wmrk4::_setPatchSReacK(uint pidx, uint ridx, double kf)
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	patch->setKcst(lridx, kf);
 
@@ -560,7 +728,12 @@ bool swmrk4::Wmrk4::_getPatchSReacActive(uint pidx, uint ridx) const
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return false;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	return (patch->active(lridx));
 }
@@ -574,7 +747,12 @@ void swmrk4::Wmrk4::_setPatchSReacActive(uint pidx, uint ridx, bool a)
 	Patchdef * patch = statedef()->patchdef(pidx);
 	assert(patch != 0);
 	uint lridx = patch->sreacG2L(ridx);
-	if (lridx == ssolver::LIDX_UNDEFINED) return;
+	if (lridx == ssolver::LIDX_UNDEFINED)
+	{
+		std::ostringstream os;
+		os << "Surface reaction undefined in patch.\n";
+		throw steps::ArgErr(os.str());
+	}
 
 	patch->setActive(lridx, a);
 
@@ -588,7 +766,21 @@ double swmrk4::Wmrk4::_ccst(double kcst, double vol, uint order)
 {
     double vscale = 1.0e3 * vol * steps::math::AVOGADRO;
     int o1 = static_cast<int>(order) - 1;
-    if (o1 < 0) o1 = 0;
+    // IMPORTANT: Now treating zero-order reaction units correctly, i.e. as
+    // M/s not /s
+    // if (o1 < 0) o1 = 0;
+    return kcst * pow(vscale, static_cast<double>(-o1));
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
+double swmrk4::Wmrk4::_ccst2D(double kcst, double area, uint order)
+{
+    double vscale = area * steps::math::AVOGADRO;
+    int o1 = static_cast<int>(order) - 1;
+    // IMPORTANT: Now treating zero-order reaction units correctly, i.e. as
+    // M/s not /s
+    // if (o1 < 0) o1 = 0;
     return kcst * pow(vscale, static_cast<double>(-o1));
 }
 
@@ -723,22 +915,33 @@ void swmrk4::Wmrk4::_setup(void)
 					pUpdMtx[rowp + j][mtx_ocompidx + k] = oupd;
 				}
 			}
-			/// set scaled reaction constant
-			/// depends on volume of lhs reaction compartment
-			double vol;
-			if (statedef()->patchdef(i)->sreacdef(j)->inside() == true)
-			{
-				assert(statedef()->patchdef(i)->icompdef() != 0);
-				vol = statedef()->patchdef(i)->icompdef()->vol();
+			if (statedef()->patchdef(i)->sreacdef(j)->surf_surf() == false)
+			{	/// set scaled reaction constant
+				/// depends on volume of lhs reaction compartment
+				double vol;
+				if (statedef()->patchdef(i)->sreacdef(j)->inside() == true)
+				{
+					assert(statedef()->patchdef(i)->icompdef() != 0);
+					vol = statedef()->patchdef(i)->icompdef()->vol();
+				}
+				else
+				{
+					assert(statedef()->patchdef(i)->ocompdef() != 0);
+					vol = statedef()->patchdef(i)->ocompdef()->vol();
+				}
+				double sreac_kcst = statedef()->patchdef(i)->kcst(j);
+				uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
+				pCcst.push_back(_ccst(sreac_kcst, vol, sreac_order));
 			}
 			else
 			{
-				assert(statedef()->patchdef(i)->ocompdef() != 0);
-				vol = statedef()->patchdef(i)->ocompdef()->vol();
+				/// 2D reaction
+				double area = statedef()->patchdef(i)->area();
+				double sreac_kcst = statedef()->patchdef(i)->kcst(j);
+				uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
+				pCcst.push_back(_ccst2D(sreac_kcst, area, sreac_order));
 			}
-			double sreac_kcst = statedef()->patchdef(i)->kcst(j);
-			uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
-			pCcst.push_back(_ccst(sreac_kcst, vol, sreac_order));
+
 		}
 		/// move markers to next point in matrix
 		rowp += patchReacs_N;
@@ -818,7 +1021,7 @@ void swmrk4::Wmrk4::_refillCcst(void)
 	for (uint i=0; i< Comps_N; ++i)
 	{
 		uint compReacs_N = statedef()->compdef(i)->countReacs();
-		uint compSpecs_N = statedef()->compdef(i)->countSpecs();
+		// uint compSpecs_N = statedef()->compdef(i)->countSpecs();
 
         for(uint j=0; j< compReacs_N; ++j)
 		{
@@ -837,30 +1040,41 @@ void swmrk4::Wmrk4::_refillCcst(void)
 	for(uint i=0; i< Patches_N; ++i)
 	{
 		uint patchReacs_N = statedef()->patchdef(i)->countSReacs();
-		uint patchSpecs_N_S = statedef()->patchdef(i)->countSpecs();
-	    uint patchSpecs_N_I = statedef()->patchdef(i)->countSpecs_I();
-		uint patchSpecs_N_O = statedef()->patchdef(i)->countSpecs_O();
+		// uint patchSpecs_N_S = statedef()->patchdef(i)->countSpecs();
+	    // uint patchSpecs_N_I = statedef()->patchdef(i)->countSpecs_I();
+		// uint patchSpecs_N_O = statedef()->patchdef(i)->countSpecs_O();
 
 		for (uint j=0; j< patchReacs_N; ++j)
 		{
-			/// set scaled reaction constant
-			/// depends on volume of lhs reaction compartment
-			double vol;
-			if (statedef()->patchdef(i)->sreacdef(j)->inside() == true)
+			if (statedef()->patchdef(i)->sreacdef(j)->surf_surf() == false)
 			{
-				assert(statedef()->patchdef(i)->icompdef() != 0);
-				vol = statedef()->patchdef(i)->icompdef()->vol();
-			}
+				/// set scaled reaction constant
+				/// depends on volume of lhs reaction compartment
+				double vol;
+				if (statedef()->patchdef(i)->sreacdef(j)->inside() == true)
+				{
+					assert(statedef()->patchdef(i)->icompdef() != 0);
+					vol = statedef()->patchdef(i)->icompdef()->vol();
+				}
+				else
+				{
+					assert(statedef()->patchdef(i)->ocompdef() != 0);
+					vol = statedef()->patchdef(i)->ocompdef()->vol();
+				}
+				// DEBUG 8/4/09: reaction constants were found from model level objects
+				// so didn't take into account sim-level changes
+				double sreac_kcst = statedef()->patchdef(i)->kcst(j);
+				uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
+				pCcst.push_back(_ccst(sreac_kcst, vol, sreac_order));
+				}
 			else
 			{
-				assert(statedef()->patchdef(i)->ocompdef() != 0);
-				vol = statedef()->patchdef(i)->ocompdef()->vol();
+				/// 2D reaction
+				double area = statedef()->patchdef(i)->area();
+				double sreac_kcst = statedef()->patchdef(i)->kcst(j);
+				uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
+				pCcst.push_back(_ccst2D(sreac_kcst, area, sreac_order));
 			}
-        	// DEBUG 8/4/09: reaction constants were found from model level objects
-        	// so didn't take into account sim-level changes
-			double sreac_kcst = statedef()->patchdef(i)->kcst(j);
-			uint sreac_order = statedef()->patchdef(i)->sreacdef(j)->order();
-			pCcst.push_back(_ccst(sreac_kcst, vol, sreac_order));
 		}
 	}
 
@@ -895,6 +1109,7 @@ void swmrk4::Wmrk4::_setderivs(dVec & vals, dVec & dydx)
 				double val = vals[i];
 				switch (pReacMtx[r][i])
 				{
+					case 4: dydx_lhs_temp *= val;
 					case 3: dydx_lhs_temp *= val;
 					case 2: dydx_lhs_temp *= val;
 					case 1: dydx_lhs_temp *= val;
@@ -914,6 +1129,11 @@ void swmrk4::Wmrk4::_setderivs(dVec & vals, dVec & dydx)
 		{
 			dydx[n] += pDyDxlhs[n][r];
 		}
+		// If species is clamped the dy/dx is zero:
+		if (pSFlags[n] & Statedef::CLAMPED_POOLFLAG)
+		{
+			dydx[n] = 0.0;
+		}
 	}
 }
 
@@ -953,7 +1173,8 @@ void swmrk4::Wmrk4::_rksteps(double t1, double t2)
 		os << "dt is zero or negative. Call setDT() method.";
 		throw steps::ArgErr(os.str());
 	}
-	if (pDT >= (t2-t1))
+    // 2010_11_1 remove = to allow step()
+	if (t1 + pDT > t2 )
 	{
 		std::ostringstream os;
 		os << "dt is larger than simulation step.";
@@ -1005,7 +1226,9 @@ void swmrk4::Wmrk4::_update(void)
 		}
 		else
 		{
-			pVals[i] = pNewVals[i];
+			double newval = pNewVals[i];
+			if (newval < 0.0) newval = 0.0;
+			pVals[i] = newval;
 		}
 	}
 
@@ -1019,8 +1242,7 @@ void swmrk4::Wmrk4::_update(void)
 		uint comp_Specs_N = statedef()->compdef(i)->countSpecs();
 		for (uint j=0; j< comp_Specs_N; ++j)
 		{
-			double c = pVals[c_marker + j];
-		    statedef()->compdef(i)->setCount(j, c);
+		    statedef()->compdef(i)->setCount(j, pVals[c_marker + j]);
 		}
 		c_marker += comp_Specs_N;
 	}
@@ -1030,8 +1252,7 @@ void swmrk4::Wmrk4::_update(void)
 		uint patch_Specs_N = statedef()->patchdef(i)->countSpecs();
 		for (uint j=0; j< patch_Specs_N; ++j)
 		{
-			double c = pVals[c_marker + j];
-			statedef()->patchdef(i)->setCount(j, c);
+			statedef()->patchdef(i)->setCount(j, pVals[c_marker + j]);
 		}
 		c_marker += patch_Specs_N;
 	}
@@ -1041,5 +1262,3 @@ void swmrk4::Wmrk4::_update(void)
 
 // END
 
-
-
