@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 // STEPS - STochastic Engine for Pathway Simulation
-// Copyright (C) 2007-2009Okinawa Institute of Science and Technology, Japan.
+// Copyright (C) 2007-2011Okinawa Institute of Science and Technology, Japan.
 // Copyright (C) 2003-2006University of Antwerp, Belgium.
 //
 // See the file AUTHORS for details.
@@ -23,9 +23,9 @@
 ////////////////////////////////////////////////////////////////////////////////
 
 /*
- *  Last Changed Rev:  $Rev: 349 $
- *  Last Changed Date: $Date: 2010-06-08 10:24:51 +0900 (Tue, 08 Jun 2010) $
- *  Last Changed By:   $Author: wchen $
+ *  Last Changed Rev:  $Rev: 410 $
+ *  Last Changed Date: $Date: 2011-04-07 16:11:28 +0900 (Thu, 07 Apr 2011) $
+ *  Last Changed By:   $Author: iain $
  */
 
 #ifndef STEPS_SOLVER_WMRK4_HPP
@@ -89,6 +89,10 @@ public:
 
     void setDT(double dt);
 
+    void setRk4DT(double dt)
+    { setDT(dt); }
+
+
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      GENERAL
@@ -96,6 +100,8 @@ public:
 
     double getTime(void) const;
 
+    void checkpoint(std::string const & file_name);
+    void restore(std::string const & file_name);
     ////////////////////////////////////////////////////////////////////////
     // SOLVER STATE ACCESS:
     //      COMPARTMENT
@@ -171,6 +177,10 @@ private:
 	///
 	double _ccst(double kcst, double vol, uint order);
 
+	/// returns properly scaled reaction constant for surface-surface case
+	///
+	double _ccst2D(double kcst, double area, uint order);
+
 	/// the Runge-Kutta algorithm
 	///
 	void _rk4(double pdt);
